{*******************************************************************
 *        Copyright (c) 1997 Achim Metzen, Bonn Germany            *
 *                    Compuserve 100733,615                        *
 *                                                                 *
 *        For use read readme.txt. Distribution is forbidden       *
 *                                                                 *
 *         Email only to: Achim.Metzen@bonn.netsurf.de             *
 *                                                                 *
 *       Project: Preview and Print enhancements for Quickreport   *
 *                                                                 *
 *                                                                 *
 *     Unit: amPrvPrt.pas                                          *
 *  Version: 2.0                                                   *
 *    Datum: 01.07.97 11:30:10                                     *
 *                                                                 *
 *  Notes  : We have there after previewing all the data we need.  *
 *           Also after QuickRep.Prepare.                          *
 *           All we have to do, is to put TGraphic (TMetafile)     *
 *           from the QRPrinter pages to a new printer objects     *
 *           canvas - and just print; that's all. For this purpose *
 *           i encapsulated the TPrinter (of Delphi                *
 *           not Quickreport) class: TamPrinter.                   *
 *           If created by the TQRStdPreviewDialog it works        *
 *           automatically together.                               *
 *           The TamPrinter may be also used without setting       *
 *           TQRStdPreviewDialog to print a report.                *
 *           See copyrigth information above.                      *
 *                                                                 *
 *           Please any comments und hints to:                     *
 *           Achim.Metzen@bonn.netsurf.de.                         *
 *           I Would be very glad to get responses.                *
 *                                                                 *
 *  History:                                                       *
 *                                                                 *
 *                                                                 *
 *******************************************************************}

unit amPrvPrt;

interface

uses
{$ifdef Win32}
  Windows,ComCtrls,
{$else}
  WinTypes, WinProcs,
{$endif}
  Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, Buttons,  ExtCtrls, Menus, Printers, Dialogs, qrprntr, Quickrpt,
  QrPrev, QR2Const, amConsts;

{$B-}


Type

{TSelPrinterDlg}

  TSelPrinterDlg = class(TForm)
    OKBtn: TButton;
    Bevel1: TBevel;
    Label1: TLabel;
    PrinterBox: TComboBox;
    procedure FormShow(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
  private
    fPrinter : TPrinter;
    { Private declarations }
  public
    { Public declarations }
    Constructor NewCreate(AOwner: TComponent; APrinter: TPrinter);
  end;

TPaperEQ   = Record
    Size   : Integer;
    Length : Integer;
    Width  : Integer;
  End;

{-----------------------------------}
{TamPrinter}


TamPrinter = class(TComponent)
  private
    { Private-Deklarationen }
    fCopyright           : String;
    fVersion             : String;
    fDummy               : String;
    fQuickrep            : TQuickrep;
    fOldPaperSize        : TQRPaperSize;
    fOldOrientation      : TPrinterOrientation;
    fOldPaperLength      : Extended;
    fOldPaperWidth       : Extended;
    PrintDialog          : TPrintDialog;
    StartPage            : Integer;
    EndPage              : Integer;
    fShowPrintDialog     : Boolean;
    SelPrinterDlg        : TSelPrinterDlg;
    fSelectprinterDialog : Boolean;
    IsDevModeInit        : Boolean;
    IsPrintDlgInit       : Boolean;
    IsPrinterSel         : Boolean;
    pDev,pDrv,pPort      : Pchar;
    pSDev,pSDrv,pSPort   : Pchar;
    shDevMode            : THANDLE;
    sDevMode             : pDevMode;
    sPrintersCount       : Integer;
    PaperEQ              : TPaperEQ;
    RefreshRequest       : Boolean;
  protected
    { Protected-Deklarationen }
    Function    CompareDevMode: Boolean;
    Procedure   InitDevMode;
    Procedure   StorePrinter;
    Function    PreparePrinter : Boolean;
    Procedure   RestorePrinter;
    Function    PrinterReady : Boolean;
    Procedure   SetQuickrep(Value: TQuickrep);
    Procedure   SetShowPrintDialog(Value: Boolean);
    Procedure   SetSelectPrinterDialog(Value: Boolean);
    Function    GetaQRPrinter: TQRPrinter;
    Function    GetaQuickrep: TQuickrep;
  public
    { Public-Deklarationen }
    Procedure   SelectPrinter;
    Procedure   AdjustPaper;
    Procedure   StorePaper;
    Procedure   RestorePaper;
    Function    ExecutePrintDialog : Boolean;
    Function    StartToPrint : Boolean;
    Function    PrintMetaFile(Const MetaFile: TMetaFile): Boolean;
    Function    EndPageNewPage : Boolean;
    Function    EndPageFinishToPrint : Boolean;
    Function    Print : Boolean;
    Constructor Create(AOwner: TComponent); Override;
    Destructor  Destroy; Override;
    Property    aQRPrinter: TQRPrinter read getaQRPrinter;
  published
    { Published-Deklarationen }

    Property    aQuickrep : TQuickrep read GetaQuickrep write SetQuickrep;
    Property    Copyright: String read fCopyright write fDummy Stored False;
    Property    ShowPrintDialog : Boolean read fShowPrintDialog
                          write SetShowPrintDialog Stored True default True;
    Property    SelectPrinterDialog: Boolean read fSelectPrinterDialog
                         write SetSelectPrinterDialog Stored True;
    Property    Version: String read fVersion write fDummy Stored False;
  end;

{--------------------------------------}
{TQRStdPreviewDialog}
  TQRStdPreviewDialog = Class(TComponent)
  private
    amPrinter            : TamPrinter;
    faQuickrep           : TQuickrep;
    fShowPrintDialog     : Boolean;
    fSelectprinterDialog : Boolean;
    fCopyright           : String;
    fVersion             : String;
    fDummy               : String;
 Protected
    Procedure    RefreshPreview;
    Procedure    SetaQuickrep(Value: TQuickrep);
    Function     GetaQuickrep: TQuickrep;
    Procedure    SetShowPrintDialog(Value: Boolean);
    Procedure    SetSelectPrinterDialog(Value: Boolean);
    Procedure    CloseDialog;
    Procedure    ShowDialog(Sender: TObject);
    Procedure    EnableButtons(Enable: Boolean);
    Procedure    PrintPreview(Sender: TObject);
 Public
    RefreshRequest : Boolean;
    QRPreviewForm  : TQRStandardPreview;
    Constructor  Create(AOwner: TComponent); Override;
    Destructor   Destroy; Override;
    Procedure    NewPreview(Sender: TObject);
    procedure    NewPageAvailable(Sender: TObject; PageNum: Integer);
    procedure    NewFormCloseQuery(Sender: TObject; var CanClose: Boolean);
 Published
    Property     aQuickrep: TQuickrep read GetaQuickrep Write SetaQuickrep;
    Property     ShowPrintDialog: Boolean read fShowPrintDialog
                           write SetShowPrintDialog stored True default True;
    Property     Copyright: String read fCopyright write fDummy Stored False;
    Property     SelectPrinterDialog: Boolean read fSelectPrinterDialog
                         write SetSelectPrinterDialog Stored True Default True;
    Property     Version: String read fVersion write fDummy Stored False;
  End;



Procedure Register;

implementation

{$R *.DFM}

Const
  FirstSupVer=208;

{------------------------------------------------------------------------}
{TAMPRINTER Component}
{------------------------------------------------------------------------}

{------------------------------------------------------------------------}
{TamPrinter.Create
 Purpose: Creates the Object and sets the aQuickrep and
 ShowPrintDialog (Creates PrintDialog if True).}
{------------------------------------------------------------------------}
Constructor TamPrinter.Create(AOwner: TComponent);
Var
   FoundQReps, i : Integer;
   QRR           : TQuickrep;
Begin
   inherited Create(AOwner);
   GetMem(pDev,256);
   GetMem(pDrv,256);
   GetMem(pPort,256);
   GetMem(pSDev,256);
   GetMem(pSDrv,256);
   GetMem(pSPort,256);
   sPrintersCount:=Printer.Printers.Count;
   fCopyRight:=' by Achim Metzen 1997';
   fVersion  :=' 2.0';
   if not (cQrVersion<FirstSupVer) then SelectPrinterDialog:=True;
   ShowPrintDialog:=True;
   if not (AOwner is TQRStdPreviewDialog) then
   begin
      If (csDesigning in ComponentState) then
      Begin
         FoundQReps:=0;
         QRR:=NIL;
         With AOwner do
         Begin
           For i:=0 to (AOwner.ComponentCount -1) do
           Begin
                if Components[i] is TQuickrep then
                begin
                     QRR:= Components[i] as TQuickrep;
                     Inc(FoundQReps);
                End;
           End;
         End;
         If FoundQReps=1 then aQuickrep:=QRR;   {only set if 1 Instance found}
      End;
   End;
End;

{------------------------------------------------------------------------}
{TamPrinter.Destroy
 Purpose: Standard}
{------------------------------------------------------------------------}
Destructor TamPrinter.destroy;
Begin
   { don't free Printer because it will
    be freed automatically at Application termination and may be
    used now}
   Try
     aQuickrep:=Nil;
     ShowPrintDialog:=false;  { frees the Component if existing}
{$ifdef Win32}
     FreeMem(pDev);
     FreeMem(pDrv);
     FreeMem(pPort);
     FreeMem(pSDev);
     FreeMem(pSDrv);
     FreeMem(pSPort);
{$else}
     FreeMem(pDev,256);
     FreeMem(pDrv,256);
     FreeMem(pPort,256);
     FreeMem(pSDev,256);
     FreeMem(pSDrv,256);
     FreeMem(pSPort,256);
{$endif}
     if shDevMode<>0 then
     Begin
        GlobalUnlock(shDevMode);
        GlobalFree(shDevMode);
     End;
     if Assigned(SelPrinterDlg) and not(csDestroying in
               SelPrinterDlg.ComponentState) Then SelPrinterDlg.Release;
    Finally
     inherited Destroy;
   End;
End;

{------------------------------------------------------------------------
TamPrinter.SetQuickrep(Value: TQuickrep);
 Purpose: Standard
------------------------------------------------------------------------}
Procedure TamPrinter.SetQuickrep(Value: TQuickrep);
Begin
   fQuickrep:=Value;
End;

{------------------------------------------------------------------------
Function TamPrinter.GetaQuickrep
Purpose: intelligent search the valid Quickrep
------------------------------------------------------------------------}
Function TamPrinter.GetaQuickrep: TQuickrep;
Begin
   Result:=Nil;
   if (Owner is TQRStdPreviewDialog) Then
   begin
     if TQRStdPreviewDialog(Owner).aQuickrep<>Nil
     Then Result:=TQRStdPreviewDialog(Owner).aQuickrep;
   End Else Result:=fQuickrep;
End;

{------------------------------------------------------------------------
Procedure TamPrinter.SetSelectPrinterDialog
Purpose: set value depending on cQRVersion
------------------------------------------------------------------------}
Procedure TamPrinter.SetSelectPrinterDialog(Value: Boolean);
Begin
  if fSelectPrinterDialog<> Value then
  Begin
     fSelectPrinterDialog:=Value;
     if (cQRVersion<FirstSupVer) and Value then
     Begin
       Application.MessageBox('This feature doesn''t help you in '+ cQrName +
          #13#10 + 'There is still a bug in getting' +
          ' printesettings to be correct !' +#13#10 +
          '(fixed in QR20h_beta)' + #13#10
          + 'So, choosing a special printer does not make any difference.' +
          #13#10 + 'Report will always be adjusted to standard Papersize A4.'
          + #13#10+ 'But you can print the report streched to other pagesizes.'
          +#13#10+ 'Sorry ...','TamPrinter/TQRstdPreviewDialog'
                  ,MB_OK or MB_APPLMODAL or  MB_ICONINFORMATION);
       fSelectPrinterDialog:=False;
     End;
  End;
End;

{------------------------------------------------------------------------
TamPrinter.SetShowPrintDialog(Value: Boolean);
 purpose: Set the Property and create or free the Dialog component.
------------------------------------------------------------------------}
Procedure TamPrinter.SetShowPrintDialog(Value: Boolean);
Begin
  if Value<>fShowPrintDialog then
  Begin
    if Value then
    begin
       if (not Assigned(PrintDialog)) or (Assigned(PrintDialog)
              and (csDestroying in PrintDialog.ComponentState))
         Then PrintDialog:=TPrintDialog.Create(Application);
    End Else
    Begin
      Try
        if fShowPrintDialog and Assigned(PrintDialog) and not (csDestroying
         in PrintDialog.ComponentState) then PrintDialog.Free;
      Finally
        PrintDialog:=NIL;
      End;
    End;
  fShowPrintDialog:=Value;
  End;
End;

{------------------------------------------------------------------------
TamPrinter.GetaQRPrinter
Purpose: dynamically find the valid QrPrinter
------------------------------------------------------------------------}
Function TamPrinter.GetaQRPrinter: TQRPrinter;
Begin
   Result:=Nil;
   if (Owner is TQRStdPreviewDialog) then
   begin
     if (TQRStdPreviewDialog(Owner).aQuickrep.QRPrinter)<>NIL
       Then Result:=
           TQRStdPreviewDialog(Owner).aQuickrep.QRPrinter
     Else
     If (TQRStdPreviewDialog(Owner).QRPreviewForm.QRPreview.QRPrinter)<>NIL
       Then Result:=
          TQRStdPreviewDialog(Owner).QRPreviewForm.QRPreview.QRPrinter;
   End Else
   If Assigned(fQuickrep) Then Result:=fQuickrep.QRPrinter;
End;


{------------------------------------------------------------------------
 TamPrinter.AdjustPaper
 Prupose: Try to set Papersize to Custom if not supported by printer
------------------------------------------------------------------------}
Procedure  TamPrinter.AdjustPaper;
Begin
     If (aQuickrep=NIL) or (aQRPrinter=NIL) then Exit;
     aQuickrep.PrinterSettings.PrinterIndex:=Printer.PrinterIndex;
     With (aQRPrinter) do Begin
           PrinterIndex:=Printer.PrinterIndex;
           PaperSize:=aQuickrep.Page.PaperSize;
           if PaperSize<>aQuickrep.Page.PaperSize then
           Begin
              PaperSize:=Custom;
              if PaperSize=Custom Then
              Begin
                 aQuickrep.Page.PaperSize:=Custom;
                 If fOldOrientation=poLandscape then
                 Begin
                   aQuickrep.Page.Length:=fOldPaperWidth;
                   aQuickrep.Page.Width:=fOldPaperLength;
                 End;
              End Else
              Begin
                  aQuickrep.Page.PaperSize:=PaperSize;
              End;
           End;
     End;
End;

{------------------------------------------------------------------------
 TamPrinter.StorePaper
------------------------------------------------------------------------}
Procedure   TamPrinter.StorePaper;
Begin
     IsDevModeInit:=False;
     IsPrintDlgInit:=False;
     if (aQuickrep=NIL) then Exit;
     fOldOrientation:=aQuickrep.Page.Orientation;
     fOldPaperSize:=aQuickrep.Page.PaperSize;
     fOldPaperLength:=aQuickrep.Page.Length;
     fOldPaperWidth:=aQuickrep.Page.Width;
End;

{------------------------------------------------------------------------
 TamPrinter.RestorePaper
------------------------------------------------------------------------}
Procedure   TamPrinter.RestorePaper;
Begin
   if (aQuickrep=NIL) then Exit;
   aQuickrep.Page.Orientation:=fOldOrientation;
   aQuickrep.Page.PaperSize:=fOldPaperSize;
   if fOldPaperSize=Custom then
   begin
        aQuickrep.Page.Length := fOldPaperLength;
        aQuickrep.Page.Width  := fOldPaperWidth;
   End;
 End;

{------------------------------------------------------------------------
 TamPrinter.SelectPrinter
------------------------------------------------------------------------}
Procedure  TamPrinter.SelectPrinter;
Begin
     If (aQuickrep=NIL) then Exit;
     StorePaper;
     if SelectPrinterDialog
       then SelPrinterDlg:=TSelPrinterDlg.NewCreate(Application,Printer);
  Try
     If SelectPrinterDialog then SelPrinterDlg.ShowModal;
     AdjustPaper;
  Finally
     if Assigned(SelPrinterDlg) and not(csDestroying in
               SelPrinterDlg.ComponentState) Then SelPrinterDlg.Release;
     SelPrinterDlg:=Nil;
  End;
End;

{------------------------------------------------------------------------
TamPrinter.InitDevMode
Purpose: Init the deviceMode for Printer
------------------------------------------------------------------------}
Procedure   TamPrinter.InitDevMode;
Var
   hCurDevMode        : THANDLE;
   hDevMode           : THANDLE;
   CurDevMode         : pDevMode;
   DevMode            : pDevMode;

Begin
   If IsDevModeInit then Exit;
   {--------------------------------------------------------------------}
   { Get the Printersettings}
   {--------------------------------------------------------------------}

   Printer.GetPrinter(pDev,pDrv,pPort,hCurDevMode);     { Get Settings}
   if hCurDevMode=0 then                               { this is for savety,}
   Begin                                               { if new Printer}
      Printer.SetPrinter(pDev,pDrv,pPort,0);            { produce new DevMode}
      Printer.GetPrinter(pDev,pDrv,pPort,hCurDevMode);  {Get it}
   End;

   {----------------------------------------------------------------}
   { Following section is only executed at start and if aQuickrep is set}
   {----------------------------------------------------------------}
   If (aQuickrep<>NIL) and (aQRPrinter<>NIL)  Then
   Begin
      CurDevMode:=GlobalLock(hCurDevMode);               {Lock Printer DevMode}
      hDevMode:=GlobalAlloc(GHND,GlobalSize(hCurDevMode));{ Make a new DevMode}
      DevMode:=GlobalLock(hDevMode);                      { Lock the new}
      Move(CurDevMode^,DevMode^,GlobalSize(hCurDevMode)); { Copy the data}
      GlobalUnlock(hCurDevMode);                          { Unlock the Printer}
                                                          { DevMode}
     With aQuickrep, aQRPrinter, DevMode^ do
     Begin
      {--------------------------------------------------------------}
      {get some setting from aQuickrep direct !}
        if (DM_DUPLEX = dmFields and DM_DUPLEX)
           then if PrinterSettings.Duplex then dmDuplex:=DMDUP_HORIZONTAL;
        if (DM_DEFAULTSOURCE = dmFields and DM_DEFAULTSOURCE)
            then dmDefaultSource:=Ord(PrinterSettings.OutputBin);
        if (DM_Copies = dmFields and DM_Copies)
            then dmCopies:=PrinterSettings.Copies
        Else Copies:=1;
     {End;}
     {----------------------------------------------------------------}
     { here we can't do workaround because printout would differ from
       preview image}
      if (DM_PAPERSIZE = dmFields and DM_PAPERSIZE) then
      Begin
         If PaperSize=Custom then
         Begin
            If (PaperLength=PaperEQ.Length) and (PaperWidth=PaperEQ.Width) and
              (PaperEQ.Size>Ord(High(TQRPaperSize))-1) then
            Begin
               dmPaperSize:=PaperEQ.Size;
            End Else
            Begin
               dmPaperSize:=DMPAPER_USER;
               if (DM_PAPERLENGTH = dmFields and DM_PAPERLENGTH) and
                    (DM_PAPERWIDTH = dmFields and DM_PAPERWIDTH)then
               Begin
                  dmPaperLength:=Paperlength;
                  dmPaperWidth:=PaperWidth;
               End;
            End;
         End Else dmPaperSize:=Ord(PaperSize);
      End;
      if (DM_Orientation = dmFields and DM_Orientation)
         then dmOrientation:=Ord(Orientation)+1;
     End;
     { now set the changed DevMode to the Printer object}
     Printer.SetPrinter(pDev,pDrv,pPort,hDevMode);
     GlobalUnlock(hDevMode);     { only unlock don't free !}
     IsDevModeInit:=True;        { signal that init is done}
   End;
End;

{------------------------------------------------------------------------
 TamPrinter.CompareDevMode
 Purpose: Check for user made changes, ask user to apply and try to adjust
------------------------------------------------------------------------}
Function TamPrinter.CompareDevMode: Boolean;
Var
   hCurDevMode        : THANDLE;
   CurDevMode         : pDevMode;
   PaperChanged       : Boolean;
   OrientationChanged : Boolean;
   ChangeIndex        : Byte;
   cPaperSize         : TQRPaperSize;
   cLength            : Extended;
   cWidth             : Extended;
   cOrientation       : TPrinterOrientation;
   MsgRes             : Word;
   xQRPrinter          : TQRPrinter;
{$ifndef win32}
   i                  : Integer;
   DeviceCapabilities : TDeviceCapabilities;
   hPaperDims         : THandle;
   PaperDims          : Pchar;
   PaperList          : Array[0..64] of Word;
   PaperCount         : Integer;
   PaperXY            : TPoint;
   {ExtDeviceMode      : TExtDeviceMode; }
   LibHandle          : THandle;
   LibName            : PChar;
   DummyDevMode       : TDevMode;
{$endif}
Label Stop;
Begin
     Result:=True;
     {----------------------------------------------------------------}
        { Function is only executed if aQuickrep, aQRPrinter is available}
     {----------------------------------------------------------------}
     If (aQRPrinter=NIL) or (aQuickrep=NIL) then exit;

     { Compare PageSize and Orientaion}
       Printer.GetPrinter(pDev,pDrv,pPort,hCurDevMode); { Get Settings}
       CurDevMode:=GlobalLock(hCurDevMode);             { Lock Printer DevMode}


       With CurDevMode^ do
       Begin
          {-------------------------------------------------------------------}
          { Init some Values}
          With aQuickrep.Page do
          Begin
             cPaperSize:=PaperSize;
             cLength:=length;
             cWidth:=Width;
             cOrientation:=Orientation;
          End;
          ChangeIndex:=0;
          Paperchanged:=False;
          OrientationChanged:=False;
          {$ifndef Win32}
             {necessary to get PaperLength and width from DeviceCapabilities.
             dmPaperlength and dmPaperWidth will not reflect the valid
             metrics in win31 if PaperSize changed only with the second page
             of the PrinterDialog;}
             {TDeviceCapabilities = function(DeviceName, Port: PChar;
              Index: Word; Output: PChar; var DevMode: TDevMode): Longint;}
          If dmPaperSize<>DMPAPER_USER Then
          begin
             {
             this does only help with a few printer
             @ExtDeviceMode:=GetProcAddress(LibHandle,'ExtDeviceMode');
             if Assigned(ExtDeviceMode)then
             Begin
                    MsgRes:=ExtDeviceMode(Application.Handle, LibHandle,
                       CurDevMode^,pDev,pPort,CurDevMode^,Nil, DM_IN_BUFFER
                                                              or DM_OUT_BUFFER);
             End;}
             PaperXY.X:=0;PaperXY.y:=0;
             Getmem(LibName,256);
             LibHandle:=0;
             Try
               StrCopy(LibName,pDrv);
               StrCat(LibName,'.DRV');
               LibHandle:=LoadLibrary(LibName);
               If LibHandle>31 then
               Begin
                  @DeviceCapabilities:=
                            GetProcAddress(LibHandle,'DeviceCapabilities');
                  if Assigned(DeviceCapabilities)then
                  Begin
                    FillChar(PaperList,SizeOf(Paperlist),#0);
                    PaperCount:=DeviceCapabilities(pDev,pPort,DC_PAPERS,
                                                      @PaperList,CurDevMode^);
                    MsgRes:=DeviceCapabilities(pDev,pPort,DC_PAPERSIZE,
                                                      PChar(0),CurDevMode^);
                    if MsgRes>0 then
                    begin
                      hPaperDims:=GlobalAlloc(GHND, MsgRes*SizeOf(TPoint));
                      PaperDims:=GlobalLock(hPaperDims);;
                      Try
                       MsgRes:=DeviceCapabilities(pDev,pPort,DC_PAPERSIZE,
                                                     PaperDims,DummyDevMode);

                       For i:=0 to PaperCount do
                       begin
                         if PaperList[i]=dmPaperSize then
                         Begin
                             PaperXY:=
                                TPoint(pPoint(PaperDims+(i*SizeOf(TPoint)))^);
                             Break;
                         End;
                       End;
                      Finally
                       GlobalUnlock(hPaperDims);
                       GlobalFree(hPaperDims);
                      End;
                    End;
                  End;
                  dmPaperLength:=PaperXY.Y;
                  dmPaperWidth:=PaperXY.X;
                End;
             Finally
               if LibHandle>0 then FreeLibrary(LibHandle);
               FreeMem(LibName,256);
             End;
          End;
          {$endif}

          {-----------------------------------------------------------------}
          { Look if something has changed, if both the Quickrep and the new}

          if dmOrientation<>ord(aQRPrinter.Orientation)+1
             then OrientationChanged:=True;

          {If the Value for PaperSize is different}
          {but check also if not the different values of Windows and
           QR can confuse the Result !!!!!!!!!
                  -> Custom = #27 but also Envelope DL = #27
                  again: Custom is !not! = dmPaper_User !!!!!}
          if not ((dmPaperSize=Ord(aQRPrinter.PaperSize))
                                    and (aQRPrinter.PaperSize<>Custom)) Then
          Begin
            { paper is USER and Custom then ignore the change in SIZE}
            If not((dmPaperSize=DMPAPER_USER) and
                                       (aQRPrinter.PaperSize=Custom)) Then
            Begin
              If not((dmPaperSize=PaperEQ.Size)and(dmPaperSize<DMPAPER_USER))Then
              Begin
                If Not((dmPaperLength=aQRPrinter.PaperLength) and (dmPaperWidth=
                  aQRPrinter.PaperWidth) and (dmPaperSize=PaperEQ.Size)) then
                Begin
                  PaperChanged:=True;
                End;
              End;
            End;
          End;
          {---------------------------------------------------------}
          { If PaperSize has changed and the new size is supported by QR}
          { then set it and the orientation                             }
          if PaperChanged and (dmPaperSize<Ord(High(TQRPaperSize))) and
             (dmPaperSize>=Ord(Low(TQRPaperSize))) Then
          Begin
             cPaperSize:=TQRPaperSize(dmPaperSize);
             cOrientation:=TPrinterOrientation(dmOrientation-1);
             ChangeIndex:=1;
          End;

          {------------------------------------------------------------}
          { Try to set the paper which is not supported by QR with setting}
          { Quickreps Papersize to Custom                                 }
          if PaperChanged and (dmPaperSize>Ord(High(TQRPaperSize))-1) then
          Begin
           xQRPrinter:=TQRPrinter.Create;
           xQRPrinter.PrinterIndex:=Printer.PrinterIndex;
           Try
             xQRPrinter.PaperSize:=Custom;
             if xQRPrinter.PaperSize=Custom then
             Begin
                cPaperSize:=Custom;
          {$ifdef Win32}
                cLength:=dmPaperlength;
                cWidth:=dmPaperWidth;
          {$else}
                cLength:=dmPaperlength / 10;
                cWidth:=dmPaperWidth / 10;
          {$endif}
                cOrientation:=TPrinterOrientation(dmOrientation-1);
                ChangeIndex:=2;
             End Else

             {---------------------------------------------------------------}
             { here we exit if Paper could not be set, means that DMPAPER_USER}
             { is not supported by the selected Printer                       }
             { if user selcts ignore, Result will be set to true, so Printing }
             { will start                                                     }
             Begin
                MsgRes:=MessageDlg(LoadStr(samPaperNotSupported)+#13#10+
                LoadStr(samIfSelectAbort)+LoadStr(samPrintAbort)+#13#10+
                LoadStr(samIfSelectIgnore)+LoadStr(samPrintStreched)
                                    ,mtConfirmation,[mbIgnore, mbAbort],0);
                Case MsgRes of
                    mrAbort : Result:=false;
                    mrIgnore: Result:=True;
                end;
             End; {Else begin}
           Finally
             xQRPrinter.Free;
           End; {Try}
          End;{if PaperChanged and (dmPaperSize>Ord(High(TQRPaperSize))-1) then}

          {-------------------------------------------------------------------}
          { Now: if Size and /or Orientation has changed while constantely    }
          { Paper is DMPAPER_USER ( Size changing  was ignored befor !)       }
          If (dmPaperSize=DMPAPER_USER) and Not PaperChanged then
          Begin
              If (aQuickrep.Page.Length<>dmPaperlength)
                  or (aQuickrep.Page.Width<>dmPaperWidth) then
              Begin
                If OrientationChanged Then
                Begin
          {$ifdef Win32}
                   cWidth:=dmPaperlength;
                   cLength:=dmPaperWidth;
          {$else}
                   cWidth:=dmPaperlength / 10;
                   cLength:=dmPaperWidth / 10;
          {$endif}
                End Else
                begin
          {$ifdef Win32}
                   cLength:=dmPaperlength;
                   cWidth:=dmPaperWidth;
          {$else}
                   cLength:=dmPaperlength / 10;
                   cWidth:=dmPaperWidth / 10;
          {$endif}
                End;
                { Set PaperChanged to give notice to the next section }
                PaperChanged:=True;
                ChangeIndex:=3;
              End;
          End;

          {-------------------------------------------------------------------}
          { Only if nothing else but orientation has changed }
          if OrientationChanged and not PaperChanged then
          Begin
             cOrientation:=TPrinterOrientation(dmOrientation-1);
             ChangeIndex:=4;
          End;

       End; { With CurDevMode^, aQRPrinter, aQuickrep do Begin}



       {---------------------------------------------------------------------}
       { Ask User wether to change Quickrep settings, abort or print anyway}
       { If something has changed and paper could be set in a way }

       {Following is because of bug in QR20g}
       if cQRVersion<FirstSupVer then
       Begin
         Case ChangeIndex Of
          1,2,3: Begin
                   ChangeIndex:=0;
                   MsgRes:=MessageDlg(LoadStr(samPaperNotSupported)+#13#10+
                   LoadStr(samIfSelectAbort)+LoadStr(samPrintAbort)+#13#10+
                   LoadStr(samIfSelectIgnore)+LoadStr(samPrintStreched)
                                    ,mtConfirmation,[mbIgnore, mbAbort],0);
                   Case MsgRes of
                      mrAbort : Begin
                                  Result:=false;
                                  IsDevModeInit:=False;
                                End;
                      mrIgnore: Result:=True;
                   end;
                 End;
         End;
       End;

       {-------------------------------------------------------------}
       {Exit here if no changes or no changes are usefull because of bug in QR}
       If ChangeIndex=0 then Goto Stop;

       MsgRes:=MessageDlg(LoadStr(samPrevPaperNoMatch)+ #13#10 +
               LoadStr(samIfSelectAbort)+LoadStr(samPrintAbort)+#13#10+
               LoadStr(samIfSelectRetry)+LoadStr(samRenewPreview)+#13#10+
               LoadStr(samIfSelectIgnore)+LoadStr(samPrintStreched)
                                 ,mtConfirmation, MBABORTRETRYIGNORE ,0);
       Case MsgRes of
          mrAbort: Result:=False;

          mrRetry: Begin
                     Result:=False;
                     aQuickrep.PrinterSettings.PrinterIndex
                                            :=Printer.PrinterIndex;
                     With aQuickrep do
                     Begin
                       Case ChangeIndex of
                         1: Begin
                              Page.PaperSize:=cPaperSize;
                              Page.Orientation:=cOrientation;
                            End;
                         2: Begin
                              With PaperEQ do
                              begin
                                Size:=CurdevMode^.dmPaperSize;
                                Length:=CurdevMode^.dmPaperLength;
                                Width:=CurDevMode^.dmPaperWidth;
                              End;
                              Page.PaperSize:=cPaperSize;
                              Page.length:=cLength;
                              Page.Width:=cWidth;
                              Page.Orientation:=cOrientation;
                            End;
                         3: Begin
                              Page.length:=cLength;
                              Page.Width:=cWidth;
                            End;
                         4: Page.Orientation:=cOrientation;
                       End;
                       IsPrintDlgInit:=False;
                       IsDevModeInit:=False;
                     End;
                   if (Owner is TQRStdPreviewDialog) then
                     TQRStdPreviewDialog(Self.Owner).RefreshRequest:=True
                   Else RefreshRequest:=True;

                   End;
       End; {Case}
Stop:
   GlobalUnlock(hCurDevMode);
End;


{------------------------------------------------------------------------}
 {TamPrinter.PreparePrinter
  purpose: Set the DeviceMode for the Printer and call PrinterDialog
   befor printing;}
{------------------------------------------------------------------------}
Function TamPrinter.PreparePrinter: Boolean;
begin
   if not IsDevModeInit then InitDevMode;
   {------------------------------------------------------------------------}
   { Following section is only ShowPrintDialog is True
   {------------------------------------------------------------------------}
   if ShowPrintDialog then                   { if property is true }
   Begin
      Result:=ExecutePrintDialog;        {--> Execute printdialog}
   End Else Result:=true;
   If Result Then Result:=CompareDevMode;
   {------------------------------------------------------------------------}
   { Only if no Printer was added store the current infos now. }
   if Printer.PrinterIndex<=sPrintersCount-1      {PrinterIndex in range  }
      Then StorePrinter;
end;

{------------------------------------------------------------------------}
{Procedure   TamPrinter.StorePrinter;
}
{------------------------------------------------------------------------}
Procedure   TamPrinter.StorePrinter;
Var
   hCurDevMode        : THANDLE;
   CurDevMode         : pDevMode;
Begin
   { Save current DevMode to sDevMode }
   if shDevMode<>0 then                        { if old is existing }
   Begin
       GlobalUnlock(shDevMode);                  { unlock old }
       GlobalFree(shDevMode);                    { free old  }
   End;
   Printer.GetPrinter(pSDev,pSDrv,pSPort,hCurDevMode);  { get settings  }
   CurDevMode:=GlobalLock(hCurDevMode);          {lock the Printer DevMode }
   shDevMode:=GlobalAlloc(GHND, GlobalSize(hCurDevMode)); {create a new  }
   sDevMode:=GlobalLock(shDevMode);                  { lock the new  }
   Move(CurDevMode^,sDevMode^,GlobalSize(hCurDevMode)); { copy data to the new }
   GlobalUnlock(hCurDevMode);                  { unlock the Printer DevMode }
End;

{------------------------------------------------------------------------}
{TamPrinter.RestorePrinter;
 purpose: If it was selected a printout to a File there will be a new Printer
   added, which will confuse Quickrep the next time previewing. So the
   Printer.PrinterIndex has to be restored after printing to a File.}
{------------------------------------------------------------------------}
Procedure TamPrinter.RestorePrinter;
Begin
   if shDevMode<>0 then
   Begin
      Printer.SetPrinter(pSDev,pSDrv,pSPort,shDevMode); {set with stored infos }
      GlobalUnlock(shDevMode);                          { unlock the stored }
      shDevMode:=0;                                     { don't free ->Printer}
   End;
End;

{------------------------------------------------------------------------}
{TamPrinter.ExecutePrintDialog
 purpose: nearly standard}
{------------------------------------------------------------------------}
Function TamPrinter.ExecutePrintDialog: Boolean;
Begin
       Result:=False;
       if not Assigned(PrintDialog) or (csDestroying
         in PrintDialog.ComponentState) then Exit;
       With PrintDialog do
       Begin
          If not IsPrintDlgInit then
          begin
            { we don't use Quickrep settings here ! }
            Options:=[poPageNums, poWarning, poHelp, poPrintToFile];
            FromPage:=StartPage;      { First page = 1}
            ToPage:=EndPage;          { last available Page of QRPrinter}
            MinPage:=1;
            MaxPage:=ToPage;
            PrintRange:=prAllPages;
            PrintToFile:=False;
            Collate:=False;
            IF (aQuickrep<>NIL) then with aQuickrep do
            Begin
              {if not(cQRVersion<FirstSupVer) then
                                       Copies:=aQuickrep.QRPrinter.Copies
               Because of a bug we get setting from aQuickrep direct
              Else}
              Copies:=aQuickrep.PrinterSettings.Copies
            End Else
            Begin
              Copies:=1;
            End;
            IsPrintDlgInit:=True;
          End;
          Result:=Execute;
          If Result and (PrintRange=prPageNums) Then
          Begin                                   { correct the print range}
            StartPage:= FromPage;
            EndPage:= ToPage;
          End;
       End;
End;


{------------------------------------------------------------------------
 TamPrinter.StartToPrint
 Purpose: Just calls the TPrinter method with some additonal checking.
------------------------------------------------------------------------}
Function TamPrinter.StartToPrint: Boolean;
Begin
  Result:=False;
  if PreparePrinter Then
  Begin
     If Printer.Printing or Printer.Aborted then Exit;
     if (aQuickrep<>NIL) then Printer.Title:=aQuickrep.ReportTitle;
     Printer.BeginDoc;
     Result:=PrinterReady;
  End;
End;

{------------------------------------------------------------------------}
{TamPrinter.PrintMetaFile
 Purpose: Can be used with any TMetafile Object.}
{------------------------------------------------------------------------}
Function TamPrinter.PrintMetaFile(Const MetaFile: TMetaFile): Boolean;
{$ifndef Win32}
Var
  PrintOffset: TPoint;
{$endif}
Begin
  Result:=False;
  If (not PrinterReady) or (not Assigned(MetaFile)) or
      (Printer.Canvas=NIL) then exit;
  With Printer do
  Begin
{$ifdef Win32}
     Canvas.StretchDraw(Rect(0,0,PageWidth,PageHeight),MetaFile);
{$else}
     SetMapMode(Handle, MM_ANISOTROPIC);
     SetViewportExtEx(Handle, GetDeviceCaps(Handle,LOGPIXELSX),
                               GetDeviceCaps(Handle,LOGPIXELSY), nil);
     SetWindowExtEx(Handle, Screen.PixelsPerInch, Screen.PixelsPerInch, nil);
     if Escape(Handle, GETPRINTINGOFFSET, 0, Nil, @PrintOffSet)>0
        then SetViewportOrgEx(Handle, -PrintOffSet.X, -PrintOffSet.Y, nil)
        Else SetViewportOrgEx(Handle, 0, 0, nil);
     PlayMetaFile(Handle, MetaFile.Handle);
{$endif}
  End;
  Result:=PrinterReady;
End;

{------------------------------------------------------------------------}
{TamPrinter.EndPageNewPage
 Purpose: Just calls the TPrinter method with some additonal checking.
}
{------------------------------------------------------------------------}
Function TamPrinter.EndPageNewPage: Boolean;
Begin
  Result:=False;
  if not PrinterReady then exit;
  Printer.NewPage;
  Result:=PrinterReady;
End;

{------------------------------------------------------------------------}
{TamPrinter.EndPageFinishToPrint
 Purpose: Just calls the TPrinter method with some additonal checking.
}
{------------------------------------------------------------------------}
Function TamPrinter.EndPageFinishToPrint:Boolean;
Begin
  Printer.EndDoc;
  if Printer.PrinterIndex>sPrintersCount-1 then RestorePrinter;
  Result:=not Printer.Printing and not Printer.Aborted;
End;

{------------------------------------------------------------------------}
{TamPrinter.PrintQRPreview
 Purpose: The main function, you see it is very simple...
 The "trick" is not to call TPrinter.NewPage if we are printing the last
 Page. That's all...
}
{------------------------------------------------------------------------}
Function TamPrinter.Print: Boolean;
Var i: Integer;
    Metafile: TMetafile;

Label Start;
Begin
  Result:=False;
  if aQuickrep=Nil then Exit;
  if not(Owner is TQRStdPreviewDialog) Then StorePaper;

Start:

  If (Owner is TQRStdPreviewDialog) Then
  begin
     TQRStdPreviewDialog(Owner).RefreshRequest:=False;
  End Else
  Begin
     RefreshRequest:=False;
     aQuickrep.Prepare;
  End;
  If (aQRPrinter=NIL) then exit;
  { do some testing if everything is ready to print }
  StartPage:=1; EndPage:=aQRPrinter.AvailablePages;
  if (aQRPrinter.Status<>mpFinished) or (EndPage<1)
     or (EndPage-StartPage<0) then Exit;
  { Begin printing }
  if not StartToPrint then
  Begin
    if not(Owner is TQRStdPreviewDialog) Then
    Begin
      if RefreshRequest then Goto Start Else
      Begin
         IsDevModeInit:=False;
         RestorePaper;
         Exit;
      End;
    End Else exit;
  End;
  For i:=StartPage to EndPage do
  Begin
    { print every single page; GetPage() has a TMetafile result }
    Metafile:=aQRPrinter.GetPage(i);
    if Metafile=Nil then Break;
    If not PrintMetaFile(Metafile) then
    Begin
      Break { may be the user did abort printing}
    end else
    Begin
      if i<EndPage then EndPageNewPage; { -> NewPage = finish current page and }
                                        { start a new one.                     }
                                        { ! force only beginning a new page    }
                                        { if current page is not the last one  }
    End;
  End;
  {Stop printing }
  Result:=EndPageFinishToPrint; {--> EndDoc = finish current page and  }
                                        { the whole document.  }
End;

{------------------------------------------------------------------------}
{TamPrinter.PrinterReady
 Purpose: additional checking if printer is still printing and printing
 was not aborted.
}
{------------------------------------------------------------------------}
Function TamPrinter.PrinterReady: Boolean;
Begin
    Result:=Printer.Printing and not Printer.Aborted;
End;


{------------------------------------------------------------------------
 TQRSTDPREVIEW DIALOG
------------------------------------------------------------------------}

{------------------------------------------------------------------------
Destructor TQRStdPreviewDialog.Create;
 Purpose: Standard and
 link aQuickrep automatically at design time.
 Search the Owner Components if there is one Quickrep Component
 and link to it. If more then one available, skip them all.
------------------------------------------------------------------------}
Constructor TQRStdPreviewDialog.Create(AOwner: TComponent);
Var
   FoundQReps, i : Integer;
   QRR           : TQuickrep;
Begin
   Inherited Create(AOwner);
   fCopyRight:=' by Achim Metzen 1997';
   fVersion  :=' 2.0';
   If (csDesigning in ComponentState) then
   Begin
      FoundQReps:=0;
      QRR:=NIL;
      With AOwner do
      Begin
           For i:=0 to (AOwner.ComponentCount -1) do
           Begin
                if Components[i] is TQuickrep then
                begin
                     QRR:= Components[i] as TQuickrep;
                     Inc(FoundQReps);
                End;
           End;
      End;
      If FoundQReps=1 then aQuickrep:=QRR;   {only set if 1 Instance found}
   End;
   amPrinter:=TamPrinter.Create(Self);
   ShowPrintDialog:=True;
   if not (cQRVersion<FirstSupVer) then SelectPrinterDialog:=True;

End;

{------------------------------------------------------------------------}
{Destructor TQRStdPreviewDialog.Destroy;
 Purpose:
 Unlink aQuickrep and Close (Destroy) PreviewForm if existing
 Destroy the amPrinter Object}
{------------------------------------------------------------------------}
Destructor TQRStdPreviewDialog.Destroy;
Begin
   Try
     aQuickrep:=NIL; {leads allso to unlink eventhandling}
     CloseDialog;    {Destroy PrevieForm if still existing}
     if Assigned(amPrinter) then amPrinter.Free;
   Finally
     amPrinter:=NIL;
     Inherited Destroy;
   End;
End;

{------------------------------------------------------------------------}
{Procedure TQRStdPreviewDialog.SetShowPrintDialog(Value: Boolean);
Purpose: Default and set the amPrinter Property}
{------------------------------------------------------------------------}
Procedure TQRStdPreviewDialog.SetShowPrintDialog(Value: Boolean);
Begin
  if Assigned(amPrinter) then
  Begin
    amPrinter.ShowPrintDialog:=Value;
  End;
  fShowPrintDialog:=Value;
End;

{------------------------------------------------------------------------}
{Procedure TQRStdPreviewDialog.SetSelectPrinterDialog(Value: Boolean);
 Purpose: Default and set the amPrinter Property}
{------------------------------------------------------------------------}
Procedure TQRStdPreviewDialog.SetSelectPrinterDialog(Value: Boolean);
Begin
  if fSelectPrinterDialog<> Value then
  Begin
     if Assigned(amPrinter) Then
     Begin
        amPrinter.SelectprinterDialog:=Value;
        fSelectPrinterDialog:=amPrinter.SelectPrinterDialog;
     End Else fSelectPrinterDialog:=Value;
  End;
End;

{------------------------------------------------------------------------}
{ TQRStdPreviewDialog.SetaQuickrep(Value: TQuickrep);
  Purpose:
  1. unlink previous OnPreview eventhandling
     (important if changing the Value at Runtime)
  2. Set the (new) value to aQuickrep property
  Eventhandling set by the Objectinspector will not be saved nor restored !!
  So, edit TQRStdPreviewDialog.NewPreview(Sender: TObject)if nessesary.
}
{------------------------------------------------------------------------}
Procedure TQRStdPreviewDialog.SetaQuickrep(Value: TQuickrep);
Begin
   if Assigned(faQuickrep) then
   Begin
      faQuickrep.OnPreview:=NIL;
   End;
   faQuickrep:=Value;
   if Assigned(faQuickrep) then
   Begin
     faQuickrep.OnPreview:=NewPreview;
   end;
End;

{------------------------------------------------------------------------}
{ TQRStdPreviewDialog.GetaQuickrep: TQuickrep;
  Purpose:
  Standard property setting. Edit more code if needed.
}
{------------------------------------------------------------------------}
Function TQRStdPreviewDialog.GetaQuickrep: TQuickrep;
Begin
   Result:=faQuickrep;
End;

{------------------------------------------------------------------------}
{  TQRStdPreviewDialog.NewPreview(Sender: TObject);
   Purpose:
   Will be linked to the Quickreport NewPreview event by setting aQuickrep.
   Edit further code if needed.
}
{------------------------------------------------------------------------}
Procedure TQRStdPreviewDialog.NewPreview(Sender: TObject);
Begin
     ShowDialog(Sender);
End;

{------------------------------------------------------------------------}
{  TQRStdPreviewDialog.NewFormCloseQuery(Sender: TObject);
   Purpose:
   Will be linked to the PreviewForm.OnFormCloseQuery event.
   Edit further code if needed.
}
{------------------------------------------------------------------------}
procedure TQRStdPreviewDialog.NewFormCloseQuery(Sender: TObject;
                                          var CanClose: Boolean);
begin
   if Assigned(amPrinter) then amPrinter.RestorePaper;
   QRPreviewForm:=NIL;
   CanClose:=True;
end;

{------------------------------------------------------------------------}
{  TQRStdPreviewDialog.NewPageAvailable(Sender: TObject);
   Purpose:
   Will be linked to the Valid QRPreview.OnPageAvailable event.
   Enables or diables PrintButtons depending on QRPrinter.State.
   Sets amPrinter.aQrPrinter if finished. Results in some Init of
   the amPrinter.
   Edit further code if needed.
}
{------------------------------------------------------------------------}
procedure TQRStdPreviewDialog.NewPageAvailable(Sender: TObject; PageNum: Integer);
begin
  With QRPreviewForm do
  begin
    UpdateInfo;
    EnableButtons(QRPreview.QRPrinter.Status=mpFinished);
  end;
end;

{------------------------------------------------------------------------}
{TQRStdPreviewDialog.CloseDialog;
  Purpose:
  Close the DialogForm if still existing;
}
{------------------------------------------------------------------------}
Procedure TQRStdPreviewDialog.CloseDialog;
Begin
  Try
     If Assigned(QRPreviewForm)
        and not(csDestroying in QRPreviewForm.ComponentState) Then
     Begin
        QRPreviewForm.Close;
     End;
  Finally
     QRPreviewForm:=NIL;
  end;
End;

{------------------------------------------------------------------------}
{TQRStdPreviewDialog.ShowDialog(Sender: TObject)
 purpose:
 Creates the StandardPreview and sets some Values; also links new
 Eventhandlers, at last shows the Form.
 }
{------------------------------------------------------------------------}
Procedure TQRStdPreviewDialog.ShowDialog(Sender: TObject);
Begin
     If Assigned(QRPreviewForm) Then CloseDialog;
     if Assigned(amPrinter) then amPrinter.SelectPrinter;
     QRPreviewForm:=
         TQRStandardPreview.CreatePreview(Application, (Sender as TQRPrinter));
  Try
     With QRPreviewForm do
     begin
       Caption:=LoadStr(SqrPreview)+': '+Caption;

       {Load some Hints from Qr2Const.res }
       ZoomToFit.Hint:=LoadStr(SqrZoomToFit);
       ZoomTo100.Hint:=LoadStr(SqrZoom100);
       ZoomToWidth.Hint:=LoadStr(SqrZoomToWidth);
       FirstPage.Hint:=LoadStr(SqrFirstPage);
       PrevPage.Hint:=LoadStr(SqrPrevPage);
       NextPage.Hint:=LoadStr(SqrNextPage);
       LastPage.Hint:=LoadStr(SqrLastPage);
       PrintSetup.Hint:=LoadStr(SqrPrinterSetup);
       Print.Hint:=LoadStr(SqrPrintReport);
       Save.Hint:=LoadStr(SqrSaveReport);
       Load.Hint:=LoadStr(SqrLoadReport);
       Exit.Hint:=LoadStr(SqrFinish)+' '+LoadStr(SqrPreview);

       { this is for international use }

       Exit.Kind:=bkClose;
       { Caption may need more space }
       If Exit.Width<90 then  Exit.Width:=90;
       { Set Status.Caption to desired language }
       {
       Status.Caption:='Bitte warten ...';
       }

       { To get the QRPeviewForm Var def. to NIL if closed by user}
       OnCloseQuery:=NewFormCloseQuery;

       { now link the new PrintButton eventhandling}
       Print.OnClick:=Self.PrintPreview;

       { this is for enabling Buttons after QRPrinter has finished}
       QRPreview.OnPageAvailable:=NewPageAvailable;

       { disable some Buttons}
       EnableButtons(False);

       { make PrintSetup unvisible because it's more confusing than it helps}
       PrintSetup.Visible:=false;

       { last not least Show the stuff}
       Show;
    End;
  Except
    CloseDialog;
    Raise;
  End;
End;

{------------------------------------------------------------------------}
{ TQRStdPreviewDialog.EnableButtons(Enable: Boolean)
  Purpose:
  Enables/Disables Buttons which to access would be no good Idea while
  preparing of Preview by Quickrep
}
{------------------------------------------------------------------------}
Procedure TQRStdPreviewDialog.EnableButtons(Enable: Boolean);
begin
  With QRPreviewForm do
  Begin
      Print.Enabled:=Enable;
      Save.Enabled:=Enable;
      Load.Enabled:=Enable;
      PrintSetup.Enabled:=Enable;
      Application.ProcessMessages;
  End;
End;


{------------------------------------------------------------------------}
{TQRStdPreviewDialog.PrintPreview(Sender: TObject)
 Purpose:
 The main thing why i wrote this sample.
 If printButton is pressed a PrintDialog will be created.
 Printer values may be set and will result in a new Preview or printing.

 Unfortunately there are a few problems with Quickreport printing.

 1. QR2.0: If use choose Quickrep for Printing, you may select certain pages
 to be print, but e.g. if you only want to print the first page, it will
 print 2 pages instead. Though the second page will be empty, it will
 cause the printer to waste one sheet of paper. Beside this, quickrep will
 prepare the job onesmore, although it was prepared befor to show the preview.

 2. The idea to use QRPreviewForm.QRPreview.QRPrinter.Print instead doesn't
 lead to preparing again, but there is no way to print only selected pages.
 It will print the whole report and - why not - a empty sheet over and above.

 prPageNums will not work correct since Qusoft would have to change
 source code of qrprntr.pas (only one Line of code, which qusoft did comment
 out !!!) and/or quickrpt.pas. I wrote them a message, but never received a
 response, so i don't know for what reasons they acted in this way.

 So, i decided to take an other way to work around:
 We have there after previewing all the data we need. All we have to do, is to
 put TGraphic (TMetafile) from the QRPrinter pages to a new printer objects
 canvas - and just print; that's all. For this purpose i derived a simple
 object which encapsulates the TPrinter (Delphi not Quickreport) and QRPrinter
 class: TamPrinter.
}
{------------------------------------------------------------------------}
procedure TQRStdPreviewDialog.PrintPreview(Sender: TObject);
begin
  Try
     If QrPreviewForm.QRPreview.QRPrinter.Status<>mpFinished then exit;
     if QRPreviewForm.Enabled then QRPreviewForm.Enabled:=False;
     amPrinter.Print;
     if RefreshRequest Then RefreshPreview;
     QRPreviewForm.Enabled:=True;
  Except
     CloseDialog;
     Raise;
  End;
end;

{------------------------------------------------------------------------}
{TQRStdPreviewDialog.RefreshPreview
 Purpose:
 This is one way to get a new preview without creating a new Previewform
 after Quickrep values have changed.
 e.g. the page orientation or size.
 }
 {------------------------------------------------------------------------}
Procedure TQRStdPreviewDialog.RefreshPreview;
Begin
 Try
  With QRPreviewForm do Begin
     EnableButtons(False);
     QRPreview.QRPrinter.Cleanup;
     QrPreview.QRPrinter.ShowingPreview:=False;
     Application.ProcessMessages;
     aQuickrep.Prepare;
     QRPreview.QRPrinter:=aQuickrep.QRPrinter;
     If ZoomTo100.Down then QRPreview.Zoom:=100;
     If ZoomToFit.Down then QRPreview.ZoomToFit;
     If ZoomToWidth.Down then QRPreview.ZoomToWidth;
     { only QRPreview.UpdateZoom; is not a good job for this purpose }
     UpdateInfo;
     EnableButtons(True);
  End;
 Except
    CloseDialog;
    Raise;
 End;
End;

{----------------------------------------------------------------------}
{ TSELPRINTERDLG Form
{------------------------------------------------------------------------}

{----------------------------------------------------------------------
TSelPrinterDlg.NewCreate
Purpose: Call inherited Ctreate and load Language Resources
------------------------------------------------------------------------}
Constructor TSelPrinterDlg.NewCreate(AOwner: TComponent; APrinter: TPrinter);
Begin
   inherited Create(AOwner);
   fPrinter:=APrinter;
   Caption:=LoadStr(samPreview)+' ...';
   Label1.Caption:=LoadStr(samSelectPrinter);
End;

{------------------------------------------------------------------------
TSelPrinterDlg.FormShow
Purpose: get the Printers form TPrinter to the Listbox Items.
------------------------------------------------------------------------}
procedure TSelPrinterDlg.FormShow(Sender: TObject);
begin
    if Assigned(fPrinter) then
    Begin
        PrinterBox.Items:=fPrinter.Printers;
        PrinterBox.ItemIndex:=fPrinter.PrinterIndex;
    End;
end;

{------------------------------------------------------------------------
TSelPrinterDlg.OKBtnClick
 Purpose: if a new printer is selected , set it.
------------------------------------------------------------------------}
procedure TSelPrinterDlg.OKBtnClick(Sender: TObject);
Var
   pDev,pDrv,pPort: Pchar;
   hCurDevMode: THANDLE;
begin
   if Assigned(fPrinter) and (fPrinter.PrinterIndex<>PrinterBox.ItemIndex) Then
   Begin
     GetMem(pDev,256);
     GetMem(pDrv,256);
     GetMem(pPort,256);
     Try
          fPrinter.PrinterIndex:=PrinterBox.ItemIndex;
          fPrinter.GetPrinter(pDev,pDrv,pPort,hCurDevMode);
          fPrinter.SetPrinter(pDev,pDrv,pPort,0);  {0 for new devMode}
     Finally
{$Ifdef Win32}
          FreeMem(pDev);
          FreeMem(pDrv);
          FreeMem(pPort);
{$else}
          FreeMem(pDev,256);
          FreeMem(pDrv,256);
          FreeMem(pPort,256);
{$endif}
     End;
   End;
end;

{------------------------------------------------------------------------}
{ Procedure REGISTER
  purpose: Standard
}
{------------------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents('QReport', [TQRStdPreviewDialog, TamPrinter]);
end;

end.
